@@ -1,34 +1,40 @@
 import os
 import shutil
+import textwrap
 import time
+from multiprocessing.pool import ThreadPool
 
 from conans.client import tools
+from conans.client.conanfile.build import run_build_method
+from conans.client.conanfile.package import run_package_method
 from conans.client.file_copier import report_copied_files
-from conans.client.generators import TXTGenerator, write_generators
+from conans.client.generators import TXTGenerator
 from conans.client.graph.graph import BINARY_BUILD, BINARY_CACHE, BINARY_DOWNLOAD, BINARY_EDITABLE, \
-    BINARY_MISSING, BINARY_SKIP, BINARY_UPDATE, BINARY_UNKNOWN
+    BINARY_MISSING, BINARY_SKIP, BINARY_UPDATE, BINARY_UNKNOWN, CONTEXT_HOST
 from conans.client.importer import remove_imports, run_imports
-from conans.client.packager import run_package_method, update_package_metadata
+from conans.client.packager import update_package_metadata
 from conans.client.recorder.action_recorder import INSTALL_ERROR_BUILDING, INSTALL_ERROR_MISSING, \
     INSTALL_ERROR_MISSING_BUILD_FOLDER
 from conans.client.source import complete_recipe_sources, config_source
+from conans.client.toolchain.base import write_toolchain
+from conans.client.tools.env import no_op
 from conans.client.tools.env import pythonpath
 from conans.errors import (ConanException, ConanExceptionInUserConanfileMethod,
                            conanfile_exception_formatter)
-from conans.model.build_info import CppInfo
-from conans.model.conan_file import get_env_context_manager
+from conans.model.build_info import CppInfo, DepCppInfo
+from conans.model.conan_file import ConanFile
 from conans.model.editable_layout import EditableLayout
 from conans.model.env_info import EnvInfo
 from conans.model.graph_info import GraphInfo
-from conans.model.graph_lock import GraphLockNode
+from conans.model.graph_lock import GraphLockFile
 from conans.model.info import PACKAGE_ID_UNKNOWN
-from conans.model.manifest import FileTreeManifest
 from conans.model.ref import PackageReference
+from conans.model.user_info import DepsUserInfo
 from conans.model.user_info import UserInfo
 from conans.paths import BUILD_INFO, CONANINFO, RUN_LOG_NAME
+from conans.util.conan_v2_mode import CONAN_V2_MODE_ENVVAR
 from conans.util.env_reader import get_env
-from conans.util.files import (clean_dirty, is_dirty, make_read_only, mkdir, rmdir, save, set_dirty,
-                               set_dirty_context_manager)
+from conans.util.files import clean_dirty, is_dirty, make_read_only, mkdir, rmdir, save, set_dirty
 from conans.util.log import logger
 from conans.util.tracer import log_package_built, log_package_got_from_local_cache
 
@@ -46,12 +52,21 @@ def build_id(conan_file):
     return None
 
 
+def add_env_conaninfo(conan_file, subtree_libnames):
+    for package_name, env_vars in conan_file._conan_env_values.data.items():
+        for name, value in env_vars.items():
+            if not package_name or package_name in subtree_libnames or \
+                    package_name == conan_file.name:
+                conan_file.info.env_values.add(name, value, package_name)
+
+
 class _PackageBuilder(object):
-    def __init__(self, cache, output, hook_manager, remote_manager):
+    def __init__(self, cache, output, hook_manager, remote_manager, generators):
         self._cache = cache
         self._output = output
         self._hook_manager = hook_manager
         self._remote_manager = remote_manager
+        self._generator_manager = generators
 
     def _get_build_folder(self, conanfile, package_layout, pref, keep_build, recorder):
         # Build folder can use a different package_ID if build_id() is defined.
@@ -64,6 +79,7 @@ class _PackageBuilder(object):
         if is_dirty(build_folder):
             self._output.warn("Build folder is dirty, removing it: %s" % build_folder)
             rmdir(build_folder)
+            clean_dirty(build_folder)
 
         # Decide if the build folder should be kept
         skip_build = conanfile.develop and keep_build
@@ -105,32 +121,26 @@ class _PackageBuilder(object):
             logger.debug("BUILD: Copied to %s", build_folder)
             logger.debug("BUILD: Files copied %s", ",".join(os.listdir(build_folder)))
 
-    def _build(self, conanfile, pref, build_folder):
+    def _build(self, conanfile, pref):
         # Read generators from conanfile and generate the needed files
         logger.info("GENERATORS: Writing generators")
-        write_generators(conanfile, build_folder, self._output)
+        self._generator_manager.write_generators(conanfile, conanfile.build_folder, self._output)
+
+        logger.info("TOOLCHAIN: Writing toolchain")
+        write_toolchain(conanfile, conanfile.build_folder, self._output)
 
         # Build step might need DLLs, binaries as protoc to generate source files
         # So execute imports() before build, storing the list of copied_files
-        copied_files = run_imports(conanfile, build_folder)
+        copied_files = run_imports(conanfile, conanfile.build_folder)
 
         try:
-            self._hook_manager.execute("pre_build", conanfile=conanfile,
-                                       reference=pref.ref, package_id=pref.id)
-            logger.debug("Call conanfile.build() with files in build folder: %s",
-                         os.listdir(build_folder))
-            self._output.highlight("Calling build()")
-            with conanfile_exception_formatter(str(conanfile), "build"):
-                conanfile.build()
-
+            run_build_method(conanfile, self._hook_manager, reference=pref.ref, package_id=pref.id)
             self._output.success("Package '%s' built" % pref.id)
-            self._output.info("Build folder %s" % build_folder)
-            self._hook_manager.execute("post_build", conanfile=conanfile,
-                                       reference=pref.ref, package_id=pref.id)
+            self._output.info("Build folder %s" % conanfile.build_folder)
         except Exception as exc:
             self._output.writeln("")
             self._output.error("Package '%s' build failed" % pref.id)
-            self._output.warn("Build folder %s" % build_folder)
+            self._output.warn("Build folder %s" % conanfile.build_folder)
             if isinstance(exc, ConanExceptionInUserConanfileMethod):
                 raise exc
             raise ConanException(exc)
@@ -189,7 +199,6 @@ class _PackageBuilder(object):
 
         # BUILD & PACKAGE
         with package_layout.conanfile_read_lock(self._output):
-            _remove_folder_raising(package_folder)
             mkdir(build_folder)
             with tools.chdir(build_folder):
                 self._output.info('Building your package in %s' % build_folder)
@@ -200,12 +209,11 @@ class _PackageBuilder(object):
                         conanfile.source_folder = build_folder
 
                     if not skip_build:
-                        with get_env_context_manager(conanfile):
-                            conanfile.build_folder = build_folder
-                            conanfile.package_folder = package_folder
-                            # In local cache, install folder always is build_folder
-                            conanfile.install_folder = build_folder
-                            self._build(conanfile, pref, build_folder)
+                        conanfile.build_folder = build_folder
+                        conanfile.package_folder = package_folder
+                        # In local cache, install folder always is build_folder
+                        conanfile.install_folder = build_folder
+                        self._build(conanfile, pref)
                         clean_dirty(build_folder)
 
                     prev = self._package(conanfile, pref, package_layout, conanfile_path,
@@ -217,8 +225,8 @@ class _PackageBuilder(object):
                     log_package_built(pref, time.time() - t1, log_file)
                     recorder.package_built(pref)
                 except ConanException as exc:
-                    recorder.package_install_error(pref, INSTALL_ERROR_BUILDING,
-                                                   str(exc), remote_name=None)
+                    recorder.package_install_error(pref, INSTALL_ERROR_BUILDING, str(exc),
+                                                   remote_name=None)
                     raise exc
 
             return node.pref
@@ -238,7 +246,9 @@ def _handle_system_requirements(conan_file, pref, cache, out):
 
     Used after remote package retrieving and before package building
     """
-    if "system_requirements" not in type(conan_file).__dict__:
+    # TODO: Check if this idiom should be generalize to all methods defined in base ConanFile
+    # Instead of calling empty methods
+    if type(conan_file).system_requirements == ConanFile.system_requirements:
         return
 
     package_layout = cache.package_layout(pref.ref)
@@ -268,25 +278,6 @@ def call_system_requirements(conanfile, output):
         raise ConanException("Error in system requirements")
 
 
-def raise_package_not_found_error(conan_file, ref, package_id, dependencies, out, recorder):
-    settings_text = ", ".join(conan_file.info.full_settings.dumps().splitlines())
-    options_text = ", ".join(conan_file.info.full_options.dumps().splitlines())
-    dependencies_text = ', '.join(dependencies)
-
-    msg = '''Can't find a '%s' package for the specified settings, options and dependencies:
-- Settings: %s
-- Options: %s
-- Dependencies: %s
-- Package ID: %s
-''' % (ref, settings_text, options_text, dependencies_text, package_id)
-    out.warn(msg)
-    recorder.package_install_error(PackageReference(ref, package_id), INSTALL_ERROR_MISSING, msg)
-    raise ConanException('''Missing prebuilt package for '%s'
-Try to build it from sources with "--build %s"
-Or read "http://docs.conan.io/en/latest/faq/troubleshooting.html#error-missing-prebuilt-package"
-''' % (ref, ref.name))
-
-
 class BinaryInstaller(object):
     """ main responsible of retrieving binary packages or building them from source
     locally in case they are not found in remotes
@@ -298,6 +289,12 @@ class BinaryInstaller(object):
         self._recorder = recorder
         self._binaries_analyzer = app.binaries_analyzer
         self._hook_manager = app.hook_manager
+        self._generator_manager = app.generator_manager
+        # Load custom generators from the cache, generators are part of the binary
+        # build and install. Generators loaded here from the cache will have precedence
+        # and overwrite possible generators loaded from packages (requires)
+        for generator_path in app.cache.generators:
+            app.loader.load_generators(generator_path)
 
     def install(self, deps_graph, remotes, build_mode, update, keep_build=False, graph_info=None):
         # order by levels and separate the root node (ref=None) from the rest
@@ -305,43 +302,136 @@ class BinaryInstaller(object):
         root_level = nodes_by_level.pop()
         root_node = root_level[0]
         # Get the nodes in order and if we have to build them
+        self._out.info("Installing (downloading, building) binaries...")
         self._build(nodes_by_level, keep_build, root_node, graph_info, remotes, build_mode, update)
 
+    @staticmethod
+    def _classify(nodes_by_level):
+        missing, downloads = [], []
+        for level in nodes_by_level:
+            for node in level:
+                if node.binary == BINARY_MISSING:
+                    missing.append(node)
+                elif node.binary in (BINARY_UPDATE, BINARY_DOWNLOAD):
+                    downloads.append(node)
+        return missing, downloads
+
+    def _raise_missing(self, missing):
+        if not missing:
+            return
+
+        missing_prefs = set(n.pref for n in missing)  # avoid duplicated
+        missing_prefs = list(sorted(missing_prefs))
+        for pref in missing_prefs:
+            self._out.error("Missing binary: %s" % str(pref))
+        self._out.writeln("")
+
+        # Report details just the first one
+        node = missing[0]
+        package_id = node.package_id
+        ref, conanfile = node.ref, node.conanfile
+        dependencies = [str(dep.dst) for dep in node.dependencies]
+
+        settings_text = ", ".join(conanfile.info.full_settings.dumps().splitlines())
+        options_text = ", ".join(conanfile.info.full_options.dumps().splitlines())
+        dependencies_text = ', '.join(dependencies)
+        requires_text = ", ".join(conanfile.info.requires.dumps().splitlines())
+
+        msg = textwrap.dedent('''\
+            Can't find a '%s' package for the specified settings, options and dependencies:
+            - Settings: %s
+            - Options: %s
+            - Dependencies: %s
+            - Requirements: %s
+            - Package ID: %s
+            ''' % (ref, settings_text, options_text, dependencies_text, requires_text, package_id))
+        conanfile.output.warn(msg)
+        self._recorder.package_install_error(PackageReference(ref, package_id),
+                                             INSTALL_ERROR_MISSING, msg)
+        missing_pkgs = "', '".join([str(pref.ref) for pref in missing_prefs])
+        if len(missing_prefs) >= 5:
+            build_str = "--build=missing"
+        else:
+            build_str = " ".join(["--build=%s" % pref.ref.name for pref in missing_prefs])
+
+        raise ConanException(textwrap.dedent('''\
+            Missing prebuilt package for '%s'
+            Try to build from sources with '%s'
+            Use 'conan search <reference> --table table.html'
+            Or read 'http://docs.conan.io/en/latest/faq/troubleshooting.html#error-missing-prebuilt-package'
+            ''' % (missing_pkgs, build_str)))
+
+    def _download(self, downloads, processed_package_refs):
+        """ executes the download of packages (both download and update), only once for a given
+        PREF, even if node duplicated
+        :param downloads: all nodes to be downloaded or updated, included repetitions
+        """
+        if not downloads:
+            return
+
+        download_nodes = []
+        for node in downloads:
+            pref = node.pref
+            if pref in processed_package_refs:
+                continue
+            processed_package_refs.add(pref)
+            assert node.prev, "PREV for %s is None" % str(node.pref)
+            download_nodes.append(node)
+
+        def _download(n):
+            npref = n.pref
+            layout = self._cache.package_layout(npref.ref, n.conanfile.short_paths)
+            # We cannot embed the package_lock inside the remote.get_package()
+            # because the handle_node_cache has its own lock
+            with layout.package_lock(pref):
+                self._download_pkg(layout, n)
+
+        parallel = self._cache.config.parallel_download
+        if parallel is not None:
+            self._out.info("Downloading binary packages in %s parallel threads" % parallel)
+            thread_pool = ThreadPool(parallel)
+            thread_pool.map(_download, [n for n in download_nodes])
+            thread_pool.close()
+            thread_pool.join()
+        else:
+            for node in download_nodes:
+                _download(node)
+
+    def _download_pkg(self, layout, node):
+        self._remote_manager.get_package(node.conanfile, node.pref, layout, node.binary_remote,
+                                         node.conanfile.output, self._recorder)
+
     def _build(self, nodes_by_level, keep_build, root_node, graph_info, remotes, build_mode, update):
+        using_build_profile = bool(graph_info.profile_build)
+        missing, downloads = self._classify(nodes_by_level)
+        self._raise_missing(missing)
         processed_package_refs = set()
+        self._download(downloads, processed_package_refs)
+
         for level in nodes_by_level:
             for node in level:
                 ref, conan_file = node.ref, node.conanfile
                 output = conan_file.output
-                package_id = node.package_id
-                if node.binary == BINARY_MISSING:
-                    dependencies = [str(dep.dst) for dep in node.dependencies]
-                    raise_package_not_found_error(conan_file, ref, package_id, dependencies,
-                                                  out=output, recorder=self._recorder)
 
-                self._propagate_info(node)
+                self._propagate_info(node, using_build_profile)
                 if node.binary == BINARY_EDITABLE:
                     self._handle_node_editable(node, graph_info)
+                    # Need a temporary package revision for package_revision_mode
+                    # Cannot be PREV_UNKNOWN otherwise the consumers can't compute their packageID
+                    node.prev = "editable"
                 else:
                     if node.binary == BINARY_SKIP:  # Privates not necessary
                         continue
                     assert ref.revision is not None, "Installer should receive RREV always"
-                    _handle_system_requirements(conan_file, node.pref, self._cache, output)
                     if node.binary == BINARY_UNKNOWN:
                         self._binaries_analyzer.reevaluate_node(node, remotes, build_mode, update)
+                        if node.binary == BINARY_MISSING:
+                            self._raise_missing([node])
+                    _handle_system_requirements(conan_file, node.pref, self._cache, output)
                     self._handle_node_cache(node, keep_build, processed_package_refs, remotes)
 
         # Finally, propagate information to root node (ref=None)
-        self._propagate_info(root_node)
-
-    @staticmethod
-    def _node_concurrently_installed(node, package_folder):
-        if node.binary == BINARY_DOWNLOAD and os.path.exists(package_folder):
-            return True
-        elif node.binary == BINARY_UPDATE:
-            read_manifest = FileTreeManifest.load(package_folder)
-            if node.update_manifest == read_manifest:
-                return True
+        self._propagate_info(root_node, using_build_profile)
 
     def _handle_node_editable(self, node, graph_info):
         # Get source of information
@@ -364,7 +454,8 @@ class BinaryInstaller(object):
             if build_folder is not None:
                 build_folder = os.path.join(base_path, build_folder)
                 output = node.conanfile.output
-                write_generators(node.conanfile, build_folder, output)
+                self._generator_manager.write_generators(node.conanfile, build_folder, output)
+                write_toolchain(node.conanfile, build_folder, output)
                 save(os.path.join(build_folder, CONANINFO), node.conanfile.info.dumps())
                 output.info("Generated %s" % CONANINFO)
                 graph_info_node = GraphInfo(graph_info.profile_host, root_ref=node.ref)
@@ -372,6 +463,10 @@ class BinaryInstaller(object):
                 graph_info_node.graph_lock = graph_info.graph_lock
                 graph_info_node.save(build_folder)
                 output.info("Generated graphinfo")
+                graph_lock_file = GraphLockFile(graph_info.profile_host, graph_info.profile_build,
+                                                graph_info.graph_lock)
+                graph_lock_file.save(os.path.join(build_folder, "conan.lock"))
+
                 save(os.path.join(build_folder, BUILD_INFO), TXTGenerator(node.conanfile).content)
                 output.info("Generated %s" % BUILD_INFO)
                 # Build step might need DLLs, binaries as protoc to generate source files
@@ -387,41 +482,32 @@ class BinaryInstaller(object):
         output = conanfile.output
 
         layout = self._cache.package_layout(pref.ref, conanfile.short_paths)
-        package_folder = layout.package(pref)
 
         with layout.package_lock(pref):
             if pref not in processed_package_references:
                 processed_package_references.add(pref)
                 if node.binary == BINARY_BUILD:
                     assert node.prev is None, "PREV for %s to be built should be None" % str(pref)
-                    with set_dirty_context_manager(package_folder):
+                    layout.package_remove(pref)
+                    with layout.set_dirty_context_manager(pref):
                         pref = self._build_package(node, output, keep_build, remotes)
                     assert node.prev, "Node PREV shouldn't be empty"
                     assert node.pref.revision, "Node PREF revision shouldn't be empty"
                     assert pref.revision is not None, "PREV for %s to be built is None" % str(pref)
                 elif node.binary in (BINARY_UPDATE, BINARY_DOWNLOAD):
-                    assert node.prev, "PREV for %s is None" % str(pref)
-                    # not really concurrently, but a different node with same pref
-                    if not self._node_concurrently_installed(node, package_folder):
-                        with set_dirty_context_manager(package_folder):
-                            assert pref.revision is not None, \
-                                "Installer should receive #PREV always"
-                            self._remote_manager.get_package(pref, package_folder,
-                                                             node.binary_remote, output,
-                                                             self._recorder)
-                            output.info("Downloaded package revision %s" % pref.revision)
-                            with layout.update_metadata() as metadata:
-                                metadata.packages[pref.id].remote = node.binary_remote.name
-                    else:
-                        output.success('Download skipped. Probable concurrent download')
-                        log_package_got_from_local_cache(pref)
-                        self._recorder.package_fetched_from_cache(pref)
+                    # this can happen after a re-evaluation of packageID with Package_ID_unknown
+                    self._download_pkg(layout, node)
                 elif node.binary == BINARY_CACHE:
                     assert node.prev, "PREV for %s is None" % str(pref)
                     output.success('Already installed!')
                     log_package_got_from_local_cache(pref)
                     self._recorder.package_fetched_from_cache(pref)
 
+            package_folder = layout.package(pref)
+            if not os.path.isdir(package_folder):
+                raise ConanException("Package '%s' corrupted. Package folder must exist: %s\n"
+                                     "Try removing the package with 'conan remove'"
+                                     % (str(pref), package_folder))
             # Call the info method
             self._call_package_info(conanfile, package_folder, ref=pref.ref)
             self._recorder.package_cpp_info(pref, conanfile.cpp_info)
@@ -438,38 +524,66 @@ class BinaryInstaller(object):
                 complete_recipe_sources(self._remote_manager, self._cache,
                                         python_require.conanfile, python_require.ref, remotes)
 
-        builder = _PackageBuilder(self._cache, output, self._hook_manager, self._remote_manager)
+        builder = _PackageBuilder(self._cache, output, self._hook_manager, self._remote_manager,
+                                  self._generator_manager)
         pref = builder.build_package(node, keep_build, self._recorder, remotes)
         if node.graph_lock_node:
-            node.graph_lock_node.modified = GraphLockNode.MODIFIED_BUILT
+            node.graph_lock_node.prev = pref.revision
         return pref
 
-    @staticmethod
-    def _propagate_info(node):
+    def _propagate_info(self, node, using_build_profile):
+        # it is necessary to recompute
+        # the node transitive information necessary to compute the package_id
+        # as it will be used by reevaluate_node() when package_revision_mode is used and
+        # PACKAGE_ID_UNKNOWN happens due to unknown revisions
+        self._binaries_analyzer.package_id_transitive_reqs(node)
         # Get deps_cpp_info from upstream nodes
         node_order = [n for n in node.public_closure if n.binary != BINARY_SKIP]
         # List sort is stable, will keep the original order of the closure, but prioritize levels
         conan_file = node.conanfile
-        transitive = node.transitive_closure.values()
+        # FIXME: Not the best place to assign the _conan_using_build_profile
+        conan_file._conan_using_build_profile = using_build_profile
+        transitive = [it for it in node.transitive_closure.values()]
+
+        br_host = []
+        for it in node.dependencies:
+            if it.require.build_require_context == CONTEXT_HOST:
+                br_host.extend(it.dst.transitive_closure.values())
+
+        # Initialize some members if we are using different contexts
+        if using_build_profile:
+            conan_file.user_info_build = DepsUserInfo()
+
         for n in node_order:
             if n not in transitive:
                 conan_file.output.info("Applying build-requirement: %s" % str(n.ref))
-            conan_file.deps_cpp_info.update(n.conanfile.cpp_info, n.ref.name)
-            conan_file.deps_env_info.update(n.conanfile.env_info, n.ref.name)
-            conan_file.deps_user_info[n.ref.name] = n.conanfile.user_info
+
+            if not using_build_profile:  # Do not touch anything
+                conan_file.deps_user_info[n.ref.name] = n.conanfile.user_info
+                conan_file.deps_cpp_info.add(n.ref.name, n.conanfile._conan_dep_cpp_info)
+                conan_file.deps_env_info.update(n.conanfile.env_info, n.ref.name)
+            else:
+                if n in transitive or n in br_host:
+                    conan_file.deps_user_info[n.ref.name] = n.conanfile.user_info
+                    conan_file.deps_cpp_info.add(n.ref.name, n.conanfile._conan_dep_cpp_info)
+                else:
+                    conan_file.user_info_build[n.ref.name] = n.conanfile.user_info
+                    env_info = EnvInfo()
+                    env_info._values_ = n.conanfile.env_info._values_.copy()
+                    # Add cpp_info.bin_paths/lib_paths to env_info (it is needed for runtime)
+                    env_info.DYLD_LIBRARY_PATH.extend(n.conanfile._conan_dep_cpp_info.lib_paths)
+                    env_info.DYLD_LIBRARY_PATH.extend(n.conanfile._conan_dep_cpp_info.framework_paths)
+                    env_info.LD_LIBRARY_PATH.extend(n.conanfile._conan_dep_cpp_info.lib_paths)
+                    env_info.PATH.extend(n.conanfile._conan_dep_cpp_info.bin_paths)
+                    conan_file.deps_env_info.update(env_info, n.ref.name)
 
         # Update the info but filtering the package values that not apply to the subtree
         # of this current node and its dependencies.
         subtree_libnames = [node.ref.name for node in node_order]
-        for package_name, env_vars in conan_file._conan_env_values.data.items():
-            for name, value in env_vars.items():
-                if not package_name or package_name in subtree_libnames or \
-                   package_name == conan_file.name:
-                    conan_file.info.env_values.add(name, value, package_name)
+        add_env_conaninfo(conan_file, subtree_libnames)
 
     def _call_package_info(self, conanfile, package_folder, ref):
-        conanfile.cpp_info = CppInfo(package_folder)
-        conanfile.cpp_info.name = conanfile.name
+        conanfile.cpp_info = CppInfo(conanfile.name, package_folder)
         conanfile.cpp_info.version = conanfile.version
         conanfile.cpp_info.description = conanfile.description
         conanfile.env_info = EnvInfo()
@@ -481,7 +595,8 @@ class BinaryInstaller(object):
         conanfile.cpp_info.public_deps = public_deps
         # Once the node is build, execute package info, so it has access to the
         # package folder and artifacts
-        with pythonpath(conanfile):  # Minimal pythonpath, not the whole context, make it 50% slower
+        conan_v2 = get_env(CONAN_V2_MODE_ENVVAR, False)
+        with pythonpath(conanfile) if not conan_v2 else no_op():  # Minimal pythonpath, not the whole context, make it 50% slower
             with tools.chdir(package_folder):
                 with conanfile_exception_formatter(str(conanfile), "package_info"):
                     conanfile.package_folder = package_folder
@@ -491,5 +606,12 @@ class BinaryInstaller(object):
                     self._hook_manager.execute("pre_package_info", conanfile=conanfile,
                                                reference=ref)
                     conanfile.package_info()
+                    if conanfile._conan_dep_cpp_info is None:
+                        try:
+                            conanfile.cpp_info._raise_incorrect_components_definition(
+                                conanfile.name, conanfile.requires)
+                        except ConanException as e:
+                            raise ConanException("%s package_info(): %s" % (str(conanfile), e))
+                        conanfile._conan_dep_cpp_info = DepCppInfo(conanfile.cpp_info)
                     self._hook_manager.execute("post_package_info", conanfile=conanfile,
                                                reference=ref)
