@@ -8,7 +8,6 @@ import uuid
 import yaml
 
 from conans.client.conf.required_version import validate_conan_version
-from conans.client.generators import registered_generators
 from conans.client.loader_txt import ConanFileTextLoader
 from conans.client.tools.files import chdir
 from conans.errors import ConanException, NotFoundException, ConanInvalidConfiguration, \
@@ -18,15 +17,16 @@ from conans.model.conan_generator import Generator
 from conans.model.options import OptionsValues
 from conans.model.ref import ConanFileReference
 from conans.model.settings import Settings
-from conans.model.values import Values
 from conans.paths import DATA_YML
 from conans.util.conan_v2_mode import CONAN_V2_MODE_ENVVAR
 from conans.util.files import load
 
 
 class ConanFileLoader(object):
-    def __init__(self, runner, output, python_requires, pyreq_loader=None):
+
+    def __init__(self, runner, output, python_requires, generator_manager=None, pyreq_loader=None):
         self._runner = runner
+        self._generator_manager = generator_manager
         self._output = output
         self._pyreq_loader = pyreq_loader
         self._python_requires = python_requires
@@ -46,13 +46,18 @@ class ConanFileLoader(object):
         """
         cached = self._cached_conanfile_classes.get(conanfile_path)
         if cached and cached[1] == lock_python_requires:
-            return cached[0](self._output, self._runner, display, user, channel), cached[2]
+            conanfile = cached[0](self._output, self._runner, display, user, channel)
+            if hasattr(conanfile, "init") and callable(conanfile.init):
+                with conanfile_exception_formatter(str(conanfile), "init"):
+                    conanfile.init()
+            return conanfile, cached[2]
 
         if lock_python_requires is not None:
             self._python_requires.locked_versions = {r.name: r for r in lock_python_requires}
         try:
             self._python_requires.valid = True
-            module, conanfile = parse_conanfile(conanfile_path, self._python_requires)
+            module, conanfile = parse_conanfile(conanfile_path, self._python_requires,
+                                                self._generator_manager)
             self._python_requires.valid = False
 
             self._python_requires.locked_versions = None
@@ -86,6 +91,22 @@ class ConanFileLoader(object):
         except ConanException as e:
             raise ConanException("Error loading conanfile at '{}': {}".format(conanfile_path, e))
 
+    def load_generators(self, conanfile_path):
+        """ Load generator classes from a module. Any non-generator classes
+        will be ignored. python_requires is not processed.
+        """
+        """ Parses a python in-memory module and adds any generators found
+            to the provided generator list
+            @param conanfile_module: the module to be processed
+            """
+        conanfile_module, module_id = _parse_conanfile(conanfile_path)
+        for name, attr in conanfile_module.__dict__.items():
+            if (name.startswith("_") or not inspect.isclass(attr) or
+                    attr.__dict__.get("__module__") != module_id):
+                continue
+            if issubclass(attr, Generator) and attr != Generator:
+                self._generator_manager.add(attr.__name__, attr, custom=True)
+
     @staticmethod
     def _load_data(conanfile_path):
         data_path = os.path.join(os.path.dirname(conanfile_path), DATA_YML)
@@ -165,7 +186,7 @@ class ConanFileLoader(object):
                         pkg_settings = settings
                         break
             if pkg_settings:
-                tmp_settings.values = Values.from_list(pkg_settings)
+                tmp_settings.update_values(pkg_settings)
 
         conanfile.initialize(tmp_settings, profile.env_values)
 
@@ -294,7 +315,7 @@ class ConanFileLoader(object):
         return conanfile
 
 
-def _parse_module(conanfile_module, module_id):
+def _parse_module(conanfile_module, module_id, generator_manager):
     """ Parses a python in-memory module, to extract the classes, mainly the main
     class defining the Recipe, but also process possible existing generators
     @param conanfile_module: the module to be processed
@@ -312,7 +333,7 @@ def _parse_module(conanfile_module, module_id):
             else:
                 raise ConanException("More than 1 conanfile in the file")
         elif issubclass(attr, Generator) and attr != Generator:
-            registered_generators.add(attr.__name__, attr, custom=True)
+            generator_manager.add(attr.__name__, attr, custom=True)
 
     if result is None:
         raise ConanException("No subclass of ConanFile")
@@ -320,11 +341,11 @@ def _parse_module(conanfile_module, module_id):
     return result
 
 
-def parse_conanfile(conanfile_path, python_requires):
+def parse_conanfile(conanfile_path, python_requires, generator_manager):
     with python_requires.capture_requires() as py_requires:
         module, filename = _parse_conanfile(conanfile_path)
         try:
-            conanfile = _parse_module(module, filename)
+            conanfile = _parse_module(module, filename, generator_manager)
 
             # Check for duplicates
             # TODO: move it into PythonRequires
