@@ -64,6 +64,7 @@ class _CppInfo(object):
         self.filenames = {}  # name of filename to create for various generators
         self.rootpath = ""
         self.sysroot = ""
+        self.requires = []
         self._build_modules_paths = None
         self._include_paths = None
         self._lib_paths = None
@@ -166,7 +167,7 @@ class _CppInfo(object):
 
 class Component(_CppInfo):
 
-    def __init__(self, rootpath):
+    def __init__(self, rootpath, version):
         super(Component, self).__init__()
         self.rootpath = rootpath
         self.includedirs.append(DEFAULT_INCLUDE)
@@ -176,6 +177,7 @@ class Component(_CppInfo):
         self.builddirs.append(DEFAULT_BUILD)
         self.frameworkdirs.append(DEFAULT_FRAMEWORK)
         self.requires = []
+        self.version = version
 
 
 class CppInfo(_CppInfo):
@@ -196,7 +198,7 @@ class CppInfo(_CppInfo):
         self.resdirs.append(DEFAULT_RES)
         self.builddirs.append(DEFAULT_BUILD)
         self.frameworkdirs.append(DEFAULT_FRAMEWORK)
-        self.components = DefaultOrderedDict(lambda: Component(self.rootpath))
+        self.components = DefaultOrderedDict(lambda: Component(self.rootpath, self.version))
         # public_deps is needed to accumulate list of deps for cmake targets
         self.public_deps = []
         self._configs = {}
@@ -240,52 +242,77 @@ class CppInfo(_CppInfo):
         return self._configs.setdefault(config, _get_cpp_info())
 
     def _raise_incorrect_components_definition(self, package_name, package_requires):
+        if not self.components and not self.requires:
+            return
+
         # Raise if mixing components
-        if (self.includedirs != [DEFAULT_INCLUDE] or
-            self.libdirs != [DEFAULT_LIB] or
-            self.bindirs != [DEFAULT_BIN] or
-            self.resdirs != [DEFAULT_RES] or
-            self.builddirs != [DEFAULT_BUILD] or
-            self.frameworkdirs != [DEFAULT_FRAMEWORK] or
-            self.libs or
-            self.system_libs or
-            self.frameworks or
-            self.defines or
-            self.cflags or
-            self.cxxflags or
-            self.sharedlinkflags or
-            self.exelinkflags or
-            self.build_modules) and self.components:
+        if self.components and \
+            (self.includedirs != [DEFAULT_INCLUDE] or
+             self.libdirs != [DEFAULT_LIB] or
+             self.bindirs != [DEFAULT_BIN] or
+             self.resdirs != [DEFAULT_RES] or
+             self.builddirs != [DEFAULT_BUILD] or
+             self.frameworkdirs != [DEFAULT_FRAMEWORK] or
+             self.libs or
+             self.system_libs or
+             self.frameworks or
+             self.defines or
+             self.cflags or
+             self.cxxflags or
+             self.sharedlinkflags or
+             self.exelinkflags or
+             self.build_modules or
+             self.requires):
             raise ConanException("self.cpp_info.components cannot be used with self.cpp_info "
                                  "global values at the same time")
-        if self._configs and self.components:
+        if self._configs:
             raise ConanException("self.cpp_info.components cannot be used with self.cpp_info configs"
                                  " (release/debug/...) at the same time")
 
-        # Raise on component name
-        for comp_name, comp in self.components.items():
-            if comp_name == package_name:
-                raise ConanException("Component name cannot be the same as the package name: '%s'"
-                                     % comp_name)
+        pkg_requires = [require.ref.name for require in package_requires.values()]
 
-        if self.components:
-            comp_requires = set()
-            for comp_name, comp in self.components.items():
-                for comp_require in comp.requires:
-                    if COMPONENT_SCOPE in comp_require:
-                        comp_requires.add(
-                            comp_require[:comp_require.find(COMPONENT_SCOPE)])
-            pkg_requires = [require.ref.name for require in package_requires.values()]
+        def _check_components_requires_instersection(comp_requires):
+            reqs = [it.split(COMPONENT_SCOPE)[0] for it in comp_requires if COMPONENT_SCOPE in it]
             # Raise on components requires without package requires
             for pkg_require in pkg_requires:
-                if pkg_require not in comp_requires:
+                if package_requires[pkg_require].private or package_requires[pkg_require].override:
+                    # Not standard requires, skip
+                    continue
+                if pkg_require not in reqs:
                     raise ConanException("Package require '%s' not used in components requires"
                                          % pkg_require)
             # Raise on components requires requiring inexistent package requires
-            for comp_require in comp_requires:
+            for comp_require in reqs:
+                reason = None
                 if comp_require not in pkg_requires:
+                    reason = "not defined as a recipe requirement"
+                elif package_requires[comp_require].private and package_requires[comp_require].override:
+                    reason = "it was defined as an overridden private recipe requirement"
+                elif package_requires[comp_require].private:
+                    reason = "it was defined as a private recipe requirement"
+                elif package_requires[comp_require].override:
+                    reason = "it was defined as an overridden recipe requirement"
+
+                if reason is not None:
                     raise ConanException("Package require '%s' declared in components requires "
-                                         "but not defined as a recipe requirement" % comp_require)
+                                         "but %s" % (comp_require, reason))
+
+        if self.components:
+            # Raise on component name
+            for comp_name, comp in self.components.items():
+                if comp_name == package_name:
+                    raise ConanException(
+                        "Component name cannot be the same as the package name: '%s'"
+                        % comp_name)
+
+            # check that requires are used in components and check that components exists in requires
+            requires_from_components = set()
+            for comp_name, comp in self.components.items():
+                requires_from_components.update(comp.requires)
+
+            _check_components_requires_instersection(requires_from_components)
+        else:
+            _check_components_requires_instersection(self.requires)
 
 
 class _BaseDepsCppInfo(_CppInfo):
@@ -307,6 +334,7 @@ class _BaseDepsCppInfo(_CppInfo):
         self.libs = merge_lists(self.libs, dep_cpp_info.libs)
         self.frameworks = merge_lists(self.frameworks, dep_cpp_info.frameworks)
         self.build_modules = merge_lists(self.build_modules, dep_cpp_info.build_modules_paths)
+        self.requires = merge_lists(self.requires, dep_cpp_info.requires)
         self.rootpaths.append(dep_cpp_info.rootpath)
 
         # Note these are in reverse order
@@ -364,6 +392,7 @@ class DepCppInfo(object):
         self._cflags = None
         self._sharedlinkflags = None
         self._exelinkflags = None
+        self._requires = None
 
         self._include_paths = None
         self._lib_paths = None
@@ -418,9 +447,11 @@ class DepCppInfo(object):
 
     def _check_component_requires(self):
         for comp_name, comp in self._cpp_info.components.items():
-            if not all([require in self._cpp_info.components for require in
-                        self._filter_component_requires(comp.requires)]):
-                raise ConanException("Component '%s' declares a missing dependency" % comp_name)
+            missing_deps = [require for require in self._filter_component_requires(comp.requires)
+                            if require not in self._cpp_info.components]
+            if missing_deps:
+                raise ConanException("Component '%s' required components not found in this package: "
+                                     "%s" % (comp_name, ", ".join("'%s'" % d for d in missing_deps)))
             bad_requires = [r for r in comp.requires if r.startswith(COMPONENT_SCOPE)]
             if bad_requires:
                 msg = "Leading character '%s' not allowed in %s requires: %s. Omit it to require " \
@@ -519,6 +550,10 @@ class DepCppInfo(object):
     def exelinkflags(self):
         return self._aggregated_values("exelinkflags")
 
+    @property
+    def requires(self):
+        return self._aggregated_values("requires")
+
 
 class DepsCppInfo(_BaseDepsCppInfo):
     """ Build Information necessary to build a given conans. It contains the
